@@ -6,8 +6,10 @@
 
 extern crate num_traits;
 #[cfg(feature = "std")] extern crate std;
+#[cfg(feature = "std")] use std::error::Error;
 
 use core::cmp::Ordering;
+use core::convert::TryFrom;
 use core::ops::{Add, AddAssign, Deref, DerefMut, Div, DivAssign, Mul, MulAssign, Neg, Rem,
                RemAssign, Sub, SubAssign};
 use core::hash::{Hash, Hasher};
@@ -23,16 +25,6 @@ use num_traits::Float;
 #[cfg(not(feature = "std"))]
 use num_traits::float::FloatCore as Float;
 
-/// A wrapper around Floats providing an implementation of Ord and Hash.
-///
-/// A NaN value cannot be stored in this type.
-#[deprecated(since = "0.6.0", note = "renamed to `NotNan`")]
-pub type NotNaN<T> = NotNan<T>;
-
-/// An error indicating an attempt to construct NotNan from a NaN
-#[deprecated(since = "0.6.0", note = "renamed to `FloatIsNan`")]
-pub type FloatIsNaN = FloatIsNan;
-
 // masks for the parts of the IEEE 754 float
 const SIGN_MASK: u64 = 0x8000000000000000u64;
 const EXP_MASK: u64 = 0x7ff0000000000000u64;
@@ -48,7 +40,7 @@ const CANONICAL_ZERO_BITS: u64 = 0x0u64;
 /// to itself, in contradiction with the IEEE standard.
 #[derive(Debug, Default, Clone, Copy)]
 #[repr(transparent)]
-pub struct OrderedFloat<T: Float>(pub T);
+pub struct OrderedFloat<T>(pub T);
 
 impl<T: Float> OrderedFloat<T> {
     /// Get the value out.
@@ -109,6 +101,12 @@ impl<T: Float> PartialEq for OrderedFloat<T> {
     }
 }
 
+impl<T: Float> PartialEq<T> for OrderedFloat<T> {
+    fn eq(&self, other: &T) -> bool {
+        self.0 == *other
+    }
+}
+
 impl<T: Float> Hash for OrderedFloat<T> {
     fn hash<H: Hasher>(&self, state: &mut H) {
         if self.is_nan() {
@@ -238,7 +236,18 @@ impl<T: Float> Zero for OrderedFloat<T> {
 /// A NaN value cannot be stored in this type.
 #[derive(PartialOrd, PartialEq, Debug, Default, Clone, Copy)]
 #[repr(transparent)]
-pub struct NotNan<T: Float>(T);
+pub struct NotNan<T>(T);
+
+impl<T> NotNan<T> {
+    /// Create a NotNan value from a value that is guaranteed to not be NaN
+    ///
+    /// # Safety
+    ///
+    /// Behaviour is undefined if `val` is NaN
+    pub const unsafe fn unchecked_new(val: T) -> Self {
+        NotNan(val)
+    }
+}
 
 impl<T: Float> NotNan<T> {
     /// Create a NotNan value.
@@ -251,14 +260,6 @@ impl<T: Float> NotNan<T> {
         }
     }
 
-    /// Create a NotNan value from a value that is guaranteed to not be NaN
-    ///
-    /// Behaviour is undefined if `val` is NaN
-    pub unsafe fn unchecked_new(val: T) -> Self {
-        debug_assert!(!val.is_nan());
-        NotNan(val)
-    }
-
     /// Get the value out.
     pub fn into_inner(self) -> T {
         self.0
@@ -280,6 +281,7 @@ impl<T: Float> Ord for NotNan<T> {
     }
 }
 
+#[allow(clippy::derive_hash_xor_eq)]
 impl<T: Float> Hash for NotNan<T> {
     fn hash<H: Hasher>(&self, state: &mut H) {
         hash_float(self.as_ref(), state)
@@ -304,12 +306,25 @@ impl From<NotNan<f64>> for f64 {
     }
 }
 
-/// Creates a NotNan value from a Float.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl<T: Float> From<T> for NotNan<T> {
-    fn from(v: T) -> Self {
-        NotNan::new(v).expect("Tried to create a NotNan from a NaN")
+impl TryFrom<f32> for NotNan<f32> {
+    type Error = FloatIsNan;
+    fn try_from(v: f32) -> Result<Self, Self::Error> {
+        NotNan::new(v)
+    }
+}
+
+impl TryFrom<f64> for NotNan<f64> {
+    type Error = FloatIsNan;
+    fn try_from(v: f64) -> Result<Self, Self::Error> {
+        NotNan::new(v)
+    }
+}
+
+impl From<NotNan<f32>> for NotNan<f64> {
+    fn from(v: NotNan<f32>) -> NotNan<f64> {
+        unsafe {
+            NotNan::unchecked_new(v.0 as f64)
+        }
     }
 }
 
@@ -323,6 +338,12 @@ impl<T: Float> Deref for NotNan<T> {
 
 impl<T: Float + PartialEq> Eq for NotNan<T> {}
 
+impl<T: Float> PartialEq<T> for NotNan<T> {
+    fn eq(&self, other: &T) -> bool {
+        self.0 == *other
+    }
+}
+
 /// Adds two NotNans.
 ///
 /// Panics if the computation results in NaN
@@ -368,9 +389,9 @@ impl<T: Float + Sum> Sum for NotNan<T> {
     }
 }
 
-impl<'a, T: Float + Sum> Sum<&'a NotNan<T>> for NotNan<T> {
+impl<'a, T: Float + Sum + 'a> Sum<&'a NotNan<T>> for NotNan<T> {
     fn sum<I: Iterator<Item = &'a NotNan<T>>>(iter: I) -> Self {
-        iter.map(|v| *v).sum()
+        iter.cloned().sum()
     }
 }
 
@@ -450,9 +471,9 @@ impl<T: Float + Product> Product for NotNan<T> {
     }
 }
 
-impl<'a, T: Float + Product> Product<&'a NotNan<T>> for NotNan<T> {
+impl<'a, T: Float + Product + 'a> Product<&'a NotNan<T>> for NotNan<T> {
     fn product<I: Iterator<Item = &'a NotNan<T>>>(iter: I) -> Self {
-        iter.map(|v| *v).product()
+        iter.cloned().product()
     }
 }
 
@@ -539,7 +560,7 @@ impl<T: Float> Neg for NotNan<T> {
 pub struct FloatIsNan;
 
 #[cfg(feature = "std")]
-impl std::error::Error for FloatIsNan {
+impl Error for FloatIsNan {
     fn description(&self) -> &str {
         "NotNan constructed with NaN"
     }
@@ -661,18 +682,25 @@ pub enum ParseNotNanError<E> {
 }
 
 #[cfg(feature = "std")]
-impl<E: fmt::Debug> std::error::Error for ParseNotNanError<E> {
+impl<E: fmt::Debug + Error + 'static> Error for ParseNotNanError<E> {
     fn description(&self) -> &str {
-        return "Error parsing a not-NaN floating point value";
+        "Error parsing a not-NaN floating point value"
     }
 
-    // TODO: add an implementation of cause(). This will be breaking because it requires E: Error.
+    fn source(&self) -> Option<&(dyn Error + 'static)> {
+        match self {
+            ParseNotNanError::ParseFloatError(e) => Some(e),
+            ParseNotNanError::IsNaN => None,
+        }
+    }
 }
 
-impl<E: fmt::Debug> fmt::Display for ParseNotNanError<E> {
+impl<E: fmt::Display> fmt::Display for ParseNotNanError<E> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        // TODO: replace this with a human readable fmt. Will require E: Display.
-        <Self as fmt::Debug>::fmt(self, f)
+        match self {
+            ParseNotNanError::ParseFloatError(e) => write!(f, "Parse error: {}", e),
+            ParseNotNanError::IsNaN => write!(f, "NotNan parser encounter a NaN"),
+        }
     }
 }
 
@@ -681,7 +709,7 @@ impl<T: Float> Num for NotNan<T> {
 
     fn from_str_radix(src: &str, radix: u32) -> Result<Self, Self::FromStrRadixErr> {
         T::from_str_radix(src, radix)
-            .map_err(|err| ParseNotNanError::ParseFloatError(err))
+            .map_err(ParseNotNanError::ParseFloatError)
             .and_then(|n| NotNan::new(n).map_err(|_| ParseNotNanError::IsNaN))
     }
 }
